#!/usr/bin/env python3
#
# Dieses Dokument ist in der Zusammenarbeit von Informatikreferendaren und
# ehemaligen Informatikreferendaren der Studienseminare (heute ZfsL) Arnsberg,
# Hamm und Solingen entstanden.
#
# Das Dokument steht unter der Lizenz: Creative Commons by-nc-sa Version 4.0
# http://creativecommons.org/licenses/by-nc-sa/4.0/deed.de
#
# Nach dieser Lizenz darf das Dokument beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter gleichen Lizenzbedingungen vertrieben
# und auf die ursprünglichen Urheber verwiesen wird.
# Eine kommerzielle Nutzung ist ausdrücklich ausgeschlossen.
#
# Die Namensnennung durch einen Verweis und die Lizenzangabe der ursprünglichen
# Urheber auf den Materialien für Schülerinnen und Schüler ist erforderlich.
#
# Die Sammlung der Dokumente steht unter
# http://ddi.uni-wuppertal.de/material/materialsammlung/
# zur Verfügung.
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html
#
# $Id: list_plus.py 1372 2015-12-15 16:50:29Z humbert $
#
# die Klasse so angepasst, dass nun auch die Typen in den 
# Köpfen dargestellt werden
#


class List (object): 
	"""
	Die Klasse List:
	Objekte der Klasse List verwalten beliebig viele, linear angeordnete 
	Objekte. Auf höchstens ein Listenobjekt, aktuelles Objekt genannt, kann 
	jeweils zugegriffen werden. 
	Wenn eine Liste leer ist, vollständig durchlaufen wurde oder das aktuelle 
	Objekt am Ende der Liste gelöscht wurde, gibt es kein aktuelles Objekt. 
	Das erste oder das letzte Objekt einer Liste können durch einen Auftrag 
	zum aktuellen Objekt gemacht werden. Außerdem kann das dem aktuellen Objekt 
	folgende Listenobjekt zum neuen aktuellen Objekt werden. 
	Das aktuelle Objekt kann gelesen, verändert oder gelöscht werden. 
	Außerdem kann vor das aktuelle Objekt ein Listenobjekt eingefügt werden.
	"""

	def __init__(self):
		"""Eine leere Liste wird erzeugt."""
		self.liste= list()
		self.nrAktuellesObjekt= None # Nummerierung beginnt bei 0
		self.aktuellesObjekt= None
		

	def isEmpty(self) -> bool:
		"""Die Anfrage liefert den Wert true, wenn die Liste keine Objekte 
		enthält, sonst liefert sie den Wert false."""
		return self.liste.__len__() == 0


	def hasAccess(self) -> bool:
		"""Die Anfrage liefert den Wert true, wenn es ein aktuelles Objekt gibt,
		sonst liefert sie den Wert false."""
		return self.aktuellesObjekt != None


	def next(self):
		"""Falls die Liste nicht leer ist, es ein aktuelles Objekt gibt und 
		dieses nicht das letzte Objekt der Liste ist, wird das dem aktuellen 
		Objekt in der Liste folgende Objekt zum aktuellen Objekt, andernfalls 
		gibt es nach Ausführung des Auftrags kein aktuelles Objekt, d.h.
		hasAccess() liefert den Wert false."""
		# Bedingungen:
		# liste nicht leer: not self.isEmpty()
		# 	und
		# es gibt ein aktuelles Objekt: self.hasAccess()
		#	und
		# aktuelles Objekt ist nicht das letzte Element der Liste:
		#		not (self.nrAktuellesObjekt == self.liste.__len__()-1)
		if not self.isEmpty() and self.hasAccess() \
				and self.nrAktuellesObjekt != self.liste.__len__()-1:
			self.nrAktuellesObjekt= self.nrAktuellesObjekt+1
			self.aktuellesObjekt= self.liste[self.nrAktuellesObjekt]
		else:
			self.aktuellesObjekt= None
			self.nrAktuellesObjekt= None


	def toFirst(self):
		"""Falls die Liste nicht leer ist, wird das erste Objekt der Liste 
		aktuelles Objekt. Ist die Liste leer, geschieht nichts."""
		if not self.isEmpty():
			self.nrAktuellesObjekt= 0
			self.aktuellesObjekt= self.liste[self.nrAktuellesObjekt]
		

	def toLast(self):
		"""Falls die Liste nicht leer ist, wird das letzte Objekt der Liste 
		aktuelles Objekt. Ist die Liste leer, geschieht nichts."""
		if not self.isEmpty():
			self.nrAktuellesObjekt= self.liste.__len__()-1
			self.aktuellesObjekt= self.liste[self.nrAktuellesObjekt]


	def getContent(self) -> 'ContentType':
		"""Falls es ein aktuelles Objekt gibt (hasAccess() == true), wird das
		aktuelle Objekt zurückgegeben, andernfalls (hasAccess()==false) gibt die 
		Anfrage den Wert null zurück."""
		if self.hasAccess():
			return self.aktuellesObjekt
		else:
			return None


	def setContent(self, pContent: 'ContentType'):
		"""Falls es ein aktuelles Objekt gibt (hasAccess() == true) und pContent 
		ungleich null ist, wird das aktuelle Objekt durch pContent ersetzt. Sonst 
		bleibt die Liste unverändert."""
		if self.hasAccess() and pContent != None:
			self.aktuellesObjekt= pContent


	def append(self, pContent):
		"""Ein neues ObjektpContent  wird am Ende der Liste eingefügt. 
		Das aktuelle Objekt bleibt unverändert. 
		Wenn die Liste leer ist, wird das Objekt pContent in die Liste eingefügt 
		und es gibt weiterhin kein aktuelles Objekt (hasAccess() == false). 
		Falls pContent gleich null ist, bleibt die Liste unverändert."""
		if pContent != None: 
			self.liste.append(pContent)


	def insert(self, pContent):
		"""Falls es ein aktuelles Objekt gibt (hasAccess() == true), wird ein 
		neues Objekt vor das aktuelle Objekt in die Liste eingefügt. 
		Das aktuelle Objekt bleibt unverändert. 
		Falls die Liste leer ist und es somit kein aktuelles Objekt gibt 
		(hasAccess() == false), wird pContent in die Liste eingefügt und es gibt 
		weiterhin kein aktuelles Objekt. 
		Falls es kein aktuelles Objekt gibt (hasAccess() == false) und die 
		Liste nicht leer ist oder pContent gleich null ist, bleibt die Liste 
		unverändert."""
		if pContent == None: 
			return
		if self.hasAccess():
			self.liste.insert(self.liste.index(self.aktuellesObjekt), pContent)
			self.nrAktuellesObjekt+=1
		elif self.isEmpty():
			self.liste.append(pContent)


	def concat(self, pList):
		"""Die Liste pList wird an die Liste angehängt. Anschließend wird pList 
		eine leere Liste. Das aktuelle Objekt bleibt unverändert. Falls pList 
		null oder eine leere Liste ist, bleibt die Liste unverändert."""
		for elem in pList.liste:
			self.liste.append(elem)
		pList=None


	def remove(self):
		"""Falls es ein aktuelles Objekt gibt (hasAccess() == true), wird das 
		aktuelle Objekt gelöscht und das Objekt hinter dem gelöschten Objekt 
		wird zum aktuellen Objekt. 
		Wird das Objekt, das am Ende der Liste steht, gelöscht, 
		gibt es kein aktuelles Objekt mehr (hasAccess() == false). 
		Wenn die Liste leer ist oder es kein aktuelles Objekt gibt 
		(hasAccess() == false), bleibt die Liste unverändert."""
		if not self.hasAccess() or self.isEmpty():
			return
		if self.nrAktuellesObjekt+1 >= self.liste.__len__():
			self.nrAktuellesObjekt= None				
		self.liste.remove(self.aktuellesObjekt)
		self.aktuellesObjekt= None # prophylaktisch
		if self.nrAktuellesObjekt != None:
			self.aktuellesObjekt= self.liste[self.nrAktuellesObjekt]
		
		
	def __str__(self):	
		CODE={
			'ENDC':0,  # RESET COLOR
			'INVERT':7,
    		 }
		def termcode(num):
		    return '\033[%sm'%num
		def colorstr(astr,color):
		    return termcode(CODE[color])+astr+termcode(CODE['ENDC'])		    

		ausgabe="Liste:\n"+6*"="+"\n"
		if self.isEmpty():
			ausgabe+="ist leer"
		if not self.hasAccess():
			ausgabe+=" -- existiert -- es gibt kein aktuelles Element\n"
#		else:
#			ausgabe+="aktuelles Element Nummer "+str(self.nrAktuellesObjekt)+": "+self.aktuellesObjekt
		for obj in self.liste:
			if obj == self.aktuellesObjekt:
				ausgabe+= colorstr(str(self.liste.index(obj))+": "+obj,"INVERT") +"   "
			else:	
				ausgabe+=str(self.liste.index(obj))+": "+obj+"   " 
		if not self.isEmpty():
			ausgabe+="\n"
		return ausgabe
		
	
if __name__ == "__main__":
	ueberschrift="Selbsttest der Klasse List\n"
	print (ueberschrift+(ueberschrift.__len__()-1)*"*")
	dieVornamen= List()
	print ("Objekt »dieVornamen« der Klasse List erstellt")
	print(dieVornamen)

	einObjekt=str("Britta")
	dieVornamen.insert(einObjekt)
	print(dieVornamen)
	
	dieVornamen.toFirst()
	print(dieVornamen)

	dieVornamen.insert("Alice")
	print(dieVornamen)
	
	dieVornamen.toLast()
	print(dieVornamen)

	dieVornamen.append("Doris")
	print(dieVornamen)
	dieVornamen.toLast()
	print(dieVornamen)

	dieVornamen.insert("Elif")
	print(dieVornamen)
	dieVornamen.toLast()
	print(dieVornamen)

	dieVornamen.next()
	print(dieVornamen)
	
	dieVornamen.toFirst()
	dieVornamen.insert("Aya")
	print(dieVornamen)
	dieVornamen.next()
	print(dieVornamen)
	dieVornamen.remove()
	print(dieVornamen)
	dieVornamen.next()
	print(dieVornamen)
	dieVornamen.remove()
	print(dieVornamen)
	dieVornamen.toLast()
	print(dieVornamen)
	dieVornamen.remove()
	print(dieVornamen)

	dieJungen=List()
	print ("Objekt »dieJungen« der Klasse List erstellt")
	print(dieJungen)
	dieJungen.append("Willi")
	dieJungen.append("Walter")
	dieJungen.append("Fritz")
	print(dieJungen)
	
	dieVornamen.concat(dieJungen)
	
	print(dieVornamen)	
	dieVornamen.toFirst()
	print(dieVornamen)
	dieVornamen.remove()
	print(dieVornamen)

	dieVornamen.remove()
	print(dieVornamen)
	dieVornamen.remove()
	print(dieVornamen)
	dieVornamen.remove()
	print(dieVornamen)
	dieVornamen.remove()
	print(dieVornamen)
	
	
